@@ -23,7 +23,8 @@
 #endif
 
 #include <gst/check/gstcheck.h>
-#include <gst/farsight/fs-codec.h>
+#include "gst/farsight/fs-codec.h"
+#include "gst/farsight/fs-rtp.h"
 
 #include "testutils.h"
 
@@ -95,21 +96,16 @@ init_codec_with_three_params (void)
   fs_codec_add_optional_parameter (codec, "aa2", "bb2");
   fs_codec_add_optional_parameter (codec, "aa3", "bb3");
 
+  fs_codec_add_feedback_parameter (codec, "aa1", "bb1", "cc1");
+  fs_codec_add_feedback_parameter (codec, "aa2", "bb2", "cc2");
+  fs_codec_add_feedback_parameter (codec, "aa3", "bb3", "cc3");
+
   codec->ABI.ABI.ptime = 12;
   codec->ABI.ABI.maxptime = 12;
 
   return codec;
 }
 
-static void
-_free_codec_param (gpointer param)
-{
-  FsCodecParameter *p = param;
-  g_free (p->name);
-  g_free (p->value);
-  g_slice_free (FsCodecParameter, p);
-}
-
 GST_START_TEST (test_fscodec_are_equal_opt_params)
 {
   FsCodec *codec1;
@@ -121,19 +117,15 @@ GST_START_TEST (test_fscodec_are_equal_opt_params)
   fail_unless (fs_codec_are_equal (codec1, codec2) == TRUE,
       "Identical codecs (with params) not recognized");
 
-  _free_codec_param (g_list_first (codec1->optional_params)->data);
-  codec1->optional_params = g_list_remove (codec1->optional_params,
+  fs_codec_remove_optional_parameter (codec1,
       g_list_first (codec1->optional_params)->data);
-
   fs_codec_add_optional_parameter (codec1, "aa1", "bb1");
 
   fail_unless (fs_codec_are_equal (codec1, codec2) == TRUE,
       "Identical codecs (with params in different order 1) not recognized");
 
-  _free_codec_param (g_list_first (codec1->optional_params)->data);
-  codec1->optional_params = g_list_remove (codec1->optional_params,
+  fs_codec_remove_optional_parameter (codec1,
       g_list_first (codec1->optional_params)->data);
-
   fs_codec_add_optional_parameter (codec1, "aa2", "bb2");
 
   fail_unless (fs_codec_are_equal (codec1, codec2) == TRUE,
@@ -143,8 +135,7 @@ GST_START_TEST (test_fscodec_are_equal_opt_params)
 
   codec1 = init_codec_with_three_params ();
 
-  _free_codec_param (g_list_first (codec1->optional_params)->data);
-  codec1->optional_params = g_list_remove (codec1->optional_params,
+  fs_codec_remove_optional_parameter (codec1,
       g_list_first (codec1->optional_params)->data);
 
   fail_unless (fs_codec_are_equal (codec1, codec2) == FALSE,
@@ -155,8 +146,7 @@ GST_START_TEST (test_fscodec_are_equal_opt_params)
   fs_codec_destroy (codec1);
 
   codec1 = init_codec_with_three_params ();
-  _free_codec_param (g_list_last (codec1->optional_params)->data);
-  codec1->optional_params = g_list_remove (codec1->optional_params,
+  fs_codec_remove_optional_parameter (codec1,
       g_list_last (codec1->optional_params)->data);
 
   fail_unless (fs_codec_are_equal (codec1, codec2) == FALSE,
@@ -170,6 +160,60 @@ GST_START_TEST (test_fscodec_are_equal_opt_params)
 GST_END_TEST;
 
 
+GST_START_TEST (test_fscodec_are_equal_feedback_params)
+{
+  FsCodec *codec1;
+  FsCodec *codec2;
+
+  codec1 = init_codec_with_three_params ();
+  codec2 = init_codec_with_three_params ();
+
+  fail_unless (fs_codec_are_equal (codec1, codec2) == TRUE,
+      "Identical codecs (with params) not recognized");
+
+  fs_codec_remove_feedback_parameter (codec1,
+      g_list_first (codec1->ABI.ABI.feedback_params));
+  fs_codec_add_feedback_parameter (codec1, "aa1", "bb1", "cc1");
+
+  fail_unless (fs_codec_are_equal (codec1, codec2) == TRUE,
+      "Identical codecs (with params in different order 1) not recognized");
+
+  fs_codec_remove_feedback_parameter (codec1,
+      g_list_first (codec1->ABI.ABI.feedback_params));
+  fs_codec_add_feedback_parameter (codec1, "aa2", "bb2", "cc2");
+
+  fail_unless (fs_codec_are_equal (codec1, codec2) == TRUE,
+      "Identical codecs (with params in different order 2) not recognized");
+
+  fs_codec_destroy (codec1);
+
+  codec1 = init_codec_with_three_params ();
+
+  fs_codec_remove_feedback_parameter (codec1,
+      g_list_first (codec1->ABI.ABI.feedback_params));
+
+  fail_unless (fs_codec_are_equal (codec1, codec2) == FALSE,
+      "Did not detect removal of first parameter of first codec");
+  fail_unless (fs_codec_are_equal (codec2, codec1) == FALSE,
+      "Did not detect removal of first parameter of second codec");
+
+  fs_codec_destroy (codec1);
+
+  codec1 = init_codec_with_three_params ();
+  fs_codec_remove_feedback_parameter (codec1,
+      g_list_last (codec1->ABI.ABI.feedback_params));
+
+  fail_unless (fs_codec_are_equal (codec1, codec2) == FALSE,
+      "Did not detect removal of last parameter of first codec");
+  fail_unless (fs_codec_are_equal (codec2, codec1) == FALSE,
+      "Did not detect removal of last parameter of second codec");
+
+  fs_codec_destroy (codec1);
+  fs_codec_destroy (codec2);
+}
+GST_END_TEST;
+
+
 GST_START_TEST (test_fscodec_copy)
 {
   FsCodec *codec1 = init_codec_with_three_params ();
@@ -226,7 +270,7 @@ GST_START_TEST (test_fscodec_keyfile)
     GList *item;
     for(item = codecs; item ; item= item->next)
     {
-      GST_DEBUG ("%s", fs_codec_to_string (item->data));
+      g_debug("%s", fs_codec_to_string (item->data));
     }
   }
 #endif
@@ -234,6 +278,9 @@ GST_START_TEST (test_fscodec_keyfile)
   codec = fs_codec_new (122, "TEST1", FS_MEDIA_TYPE_AUDIO, 8001);
   codec->channels = 5;
   fs_codec_add_optional_parameter (codec, "test3", "test4");
+  fs_codec_add_feedback_parameter (codec, "aa", "bb", "cc");
+  fs_codec_add_feedback_parameter (codec, "dd", "ee", "");
+  fs_codec_add_feedback_parameter (codec, "ff", "", "");
   comparison = g_list_append (comparison, codec);
 
   codec = fs_codec_new (123, "TEST2", FS_MEDIA_TYPE_VIDEO, 8002);
@@ -263,6 +310,92 @@ GST_START_TEST (test_fscodec_keyfile)
 }
 GST_END_TEST;
 
+GST_START_TEST (test_fscodec_rtp_hdrext)
+{
+  FsRtpHeaderExtension *hdrext, *hdrext2;
+
+  hdrext = fs_rtp_header_extension_new (1, FS_DIRECTION_BOTH, "uri");
+  hdrext2 = fs_rtp_header_extension_new (1, FS_DIRECTION_BOTH, "uri");
+
+  fail_unless (fs_rtp_header_extension_are_equal (hdrext, hdrext));
+  fail_unless (fs_rtp_header_extension_are_equal (hdrext, hdrext2));
+
+  hdrext2->id = 2;
+  fail_unless (!fs_rtp_header_extension_are_equal (hdrext, hdrext2));
+
+  hdrext2->id = 1;
+  fail_unless (fs_rtp_header_extension_are_equal (hdrext, hdrext2));
+
+  hdrext2->direction = FS_DIRECTION_NONE;
+  fail_unless (!fs_rtp_header_extension_are_equal (hdrext, hdrext2));
+  fs_rtp_header_extension_destroy (hdrext2);
+
+  hdrext2 = fs_rtp_header_extension_copy (hdrext);
+
+  fail_unless (fs_rtp_header_extension_are_equal (hdrext, hdrext2));
+
+  fs_rtp_header_extension_destroy (hdrext2);
+  fs_rtp_header_extension_destroy (hdrext);
+}
+GST_END_TEST;
+
+
+GST_START_TEST (test_fscodec_rtp_hdrext_keyfile)
+{
+  GList *extensions = NULL;
+  GError *error = NULL;
+  gchar *filename = NULL;
+  FsRtpHeaderExtension *comparison;
+
+  fail_if (fs_rtp_header_extension_list_from_keyfile ("invalid-filename",
+          FS_MEDIA_TYPE_AUDIO, &error));
+  fail_if (error == NULL);
+  fail_unless (error->domain == G_FILE_ERROR);
+  g_clear_error (&error);
+
+  filename = get_fullpath ("base/test1.conf");
+  extensions = fs_rtp_header_extension_list_from_keyfile (filename,
+      FS_MEDIA_TYPE_AUDIO, &error);
+  g_free (filename);
+  fail_unless (error == NULL);
+  fail_if (extensions == NULL);
+
+  comparison = fs_rtp_header_extension_new (1, FS_DIRECTION_BOTH,
+      "http://example.com/rtp-hdrext1");
+  fail_unless (fs_rtp_header_extension_are_equal (extensions->data,
+          comparison));
+  fs_rtp_header_extension_destroy (comparison);
+
+  comparison = fs_rtp_header_extension_new (2, FS_DIRECTION_RECV,
+      "http://example.com/rtp-hdrext2");
+  fail_unless (fs_rtp_header_extension_are_equal (extensions->next->data,
+          comparison));
+  fs_rtp_header_extension_destroy (comparison);
+
+  fail_unless (extensions->next->next == NULL);
+
+  fs_rtp_header_extension_list_destroy (extensions);
+
+
+
+  filename = get_fullpath ("base/test1.conf");
+  extensions = fs_rtp_header_extension_list_from_keyfile (filename,
+      FS_MEDIA_TYPE_VIDEO, &error);
+  g_free (filename);
+  fail_unless (error == NULL);
+  fail_if (extensions == NULL);
+
+  comparison = fs_rtp_header_extension_new (1, FS_DIRECTION_BOTH,
+      "http://example.com/rtp-hdrext1");
+  fail_unless (fs_rtp_header_extension_are_equal (extensions->data,
+          comparison));
+  fs_rtp_header_extension_destroy (comparison);
+
+  fail_unless (extensions->next == NULL);
+  fs_rtp_header_extension_list_destroy (extensions);
+}
+GST_END_TEST;
+
 static Suite *
 fscodec_suite (void)
 {
@@ -274,9 +407,12 @@ fscodec_suite (void)
   tcase_add_test (tc_chain, test_fscodec_new);
   tcase_add_test (tc_chain, test_fscodec_are_equal);
   tcase_add_test (tc_chain, test_fscodec_are_equal_opt_params);
+  tcase_add_test (tc_chain, test_fscodec_are_equal_feedback_params);
   tcase_add_test (tc_chain, test_fscodec_copy);
   tcase_add_test (tc_chain, test_fscodec_null);
   tcase_add_test (tc_chain, test_fscodec_keyfile);
+  tcase_add_test (tc_chain, test_fscodec_rtp_hdrext);
+  tcase_add_test (tc_chain, test_fscodec_rtp_hdrext_keyfile);
 
   return s;
 }
