@@ -170,6 +170,12 @@ _new_local_candidate (FsStream *stream, FsCandidate *candidate)
       other_st->target->id,
       candidate->component_id);
 
+  if (other_st->stream == NULL)
+  {
+    TEST_UNLOCK ();
+    return;
+  }
+
   candidates = g_list_prepend (NULL, candidate);
   ret = fs_stream_set_remote_candidates (other_st->stream, candidates, &error);
   g_list_free (candidates);
@@ -1369,7 +1375,7 @@ GST_START_TEST (test_rtpconference_multicast_three_way_ssrc_assoc)
     return;
   g_free (mcast_addr);
 
-  max_src_pads = 3;
+  max_src_pads = 3 * 2; /* x2 because of loopbacks causing fake conflicts */
   nway_test (3, NULL, multicast_ssrc_init, "multicast", 0, NULL);
   max_src_pads = 1;
 }
