@@ -424,7 +424,9 @@ fs_nice_stream_transmitter_stop (FsStreamTransmitter *streamtransmitter)
   gststream = self->priv->gststream;
   self->priv->gststream = NULL;
   stream_id = self->priv->stream_id;
-  self->priv->stream_id = 0;
+  /* We can't unset the stream id because it gets messy fast, just leave it as
+   * is, all calls should fail anyway
+   */
   FS_NICE_STREAM_TRANSMITTER_UNLOCK (self);
 
   if (gststream)
@@ -529,9 +531,10 @@ fs_nice_stream_transmitter_set_property (GObject *object,
     case PROP_SENDING:
       FS_NICE_STREAM_TRANSMITTER_LOCK (self);
       self->priv->sending = g_value_get_boolean (value);
+      if (self->priv->gststream)
+        fs_nice_transmitter_set_sending (self->priv->transmitter,
+            self->priv->gststream, g_value_get_boolean (value));
       FS_NICE_STREAM_TRANSMITTER_UNLOCK (self);
-      fs_nice_transmitter_set_sending (self->priv->transmitter,
-          self->priv->gststream, g_value_get_boolean (value));
       break;
     case PROP_PREFERRED_LOCAL_CANDIDATES:
       self->priv->preferred_local_candidates = g_value_dup_boxed (value);
@@ -1508,9 +1511,14 @@ agent_state_changed (NiceAgent *agent,
   fs_nice_agent_add_idle (self->priv->agent, state_changed_signal_idle,
       data, free_state_changed_signal_data);
 
-  if (state == NICE_COMPONENT_STATE_READY)
-    fs_nice_transmitter_request_keyunit (self->priv->transmitter,
-        self->priv->gststream, component_id);
+  if (fs_state >= FS_STREAM_STATE_CONNECTED)
+  {
+    FS_NICE_STREAM_TRANSMITTER_LOCK (self);
+    if (self->priv->gststream)
+      fs_nice_transmitter_request_keyunit (self->priv->transmitter,
+          self->priv->gststream, component_id);
+    FS_NICE_STREAM_TRANSMITTER_UNLOCK (self);
+  }
 }
 
 
